@@ -10,4 +10,6 @@ pub mod yyjson;
 pub use buffer::*;
 pub use bytes::*;
 pub use fragment::{orjson_fragmenttype_new, Fragment};
-pub use long::{pylong_is_unsigned, pylong_is_zero};
+pub use long::pylong_is_unsigned;
+#[cfg(feature = "inline_int")]
+pub use long::{pylong_fits_in_i32, pylong_get_inline_value, pylong_is_zero};
