@@ -1,7 +1,7 @@
 // SPDX-License-Identifier: (Apache-2.0 OR MIT)
 
-use std::ffi::CStr;
-use std::ptr::NonNull;
+use core::ffi::CStr;
+use core::ptr::NonNull;
 
 pub enum SerializeError {
     DatetimeLibraryUnsupported,
@@ -17,6 +17,7 @@ pub enum SerializeError {
     DictKeyInvalidType,
     NumpyMalformed,
     NumpyNotCContiguous,
+    NumpyNotNativeEndian,
     NumpyUnsupportedDatatype,
     UnsupportedType(NonNull<pyo3_ffi::PyObject>),
 }
@@ -26,14 +27,19 @@ impl std::fmt::Display for SerializeError {
     #[cfg_attr(feature = "optimize", optimize(size))]
     fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
         match *self {
-            SerializeError::DatetimeLibraryUnsupported => write!(f, "datetime's timezone library is not supported: use datetime.timezone.utc, pendulum, pytz, or dateutil"),
+            SerializeError::DatetimeLibraryUnsupported => write!(
+                f,
+                "datetime's timezone library is not supported: use datetime.timezone.utc, pendulum, pytz, or dateutil"
+            ),
             SerializeError::DefaultRecursionLimit => {
                 write!(f, "default serializer exceeds recursion limit")
             }
             SerializeError::Integer53Bits => write!(f, "Integer exceeds 53-bit range"),
             SerializeError::Integer64Bits => write!(f, "Integer exceeds 64-bit range"),
             SerializeError::InvalidStr => write!(f, "{}", crate::util::INVALID_STR),
-            SerializeError::InvalidFragment => write!(f, "orjson.Fragment's content is not of type bytes or str"),
+            SerializeError::InvalidFragment => {
+                write!(f, "orjson.Fragment's content is not of type bytes or str")
+            }
             SerializeError::KeyMustBeStr => write!(f, "Dict key must be str"),
             SerializeError::RecursionLimit => write!(f, "Recursion limit reached"),
             SerializeError::TimeHasTzinfo => write!(f, "datetime.time must not have tzinfo set"),
@@ -48,11 +54,15 @@ impl std::fmt::Display for SerializeError {
                 f,
                 "numpy array is not C contiguous; use ndarray.tolist() in default"
             ),
+            SerializeError::NumpyNotNativeEndian => {
+                write!(f, "numpy array is not native-endianness")
+            }
             SerializeError::NumpyUnsupportedDatatype => {
                 write!(f, "unsupported datatype in numpy array")
             }
             SerializeError::UnsupportedType(ptr) => {
-                let name = unsafe { CStr::from_ptr((*ob_type!(ptr.as_ptr())).tp_name).to_string_lossy() };
+                let name =
+                    unsafe { CStr::from_ptr((*ob_type!(ptr.as_ptr())).tp_name).to_string_lossy() };
                 write!(f, "Type is not JSON serializable: {}", name)
             }
         }
