@@ -1,10 +1,9 @@
 // SPDX-License-Identifier: (Apache-2.0 OR MIT)
 
 use crate::opt::*;
+use crate::serialize::buffer::SmallFixedBuffer;
 use crate::serialize::error::SerializeError;
-use crate::serialize::per_type::datetimelike::{
-    DateTimeBuffer, DateTimeError, DateTimeLike, Offset,
-};
+use crate::serialize::per_type::datetimelike::{DateTimeError, DateTimeLike, Offset};
 #[cfg(Py_3_9)]
 use crate::typeref::ZONEINFO_TYPE;
 use crate::typeref::{CONVERT_METHOD_STR, DST_STR, NORMALIZE_METHOD_STR, UTCOFFSET_METHOD_STR};
@@ -42,7 +41,8 @@ impl Date {
         Date { ptr: ptr }
     }
 
-    pub fn write_buf(&self, buf: &mut DateTimeBuffer) {
+    #[inline(never)]
+    pub fn write_buf(&self, buf: &mut SmallFixedBuffer) {
         {
             let year = ffi!(PyDateTime_GET_YEAR(self.ptr));
             let mut yearbuf = itoa::Buffer::new();
@@ -66,15 +66,13 @@ impl Date {
     }
 }
 impl Serialize for Date {
-    #[cold]
-    #[inline(never)]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
     {
-        let mut buf = DateTimeBuffer::new();
+        let mut buf = SmallFixedBuffer::new();
         self.write_buf(&mut buf);
-        serializer.serialize_str(str_from_slice!(buf.as_ptr(), buf.len()))
+        serializer.serialize_unit_struct(str_from_slice!(buf.as_ptr(), buf.len()))
     }
 }
 
@@ -95,7 +93,8 @@ impl Time {
         }
     }
 
-    pub fn write_buf(&self, buf: &mut DateTimeBuffer) -> Result<(), TimeError> {
+    #[inline(never)]
+    pub fn write_buf(&self, buf: &mut SmallFixedBuffer) -> Result<(), TimeError> {
         if unsafe { (*(self.ptr as *mut pyo3_ffi::PyDateTime_Time)).hastzinfo == 1 } {
             return Err(TimeError::HasTimezone);
         }
@@ -116,17 +115,15 @@ impl Time {
 }
 
 impl Serialize for Time {
-    #[cold]
-    #[inline(never)]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
     {
-        let mut buf = DateTimeBuffer::new();
+        let mut buf = SmallFixedBuffer::new();
         if self.write_buf(&mut buf).is_err() {
             err!(SerializeError::DatetimeLibraryUnsupported)
         };
-        serializer.serialize_str(str_from_slice!(buf.as_ptr(), buf.len()))
+        serializer.serialize_unit_struct(str_from_slice!(buf.as_ptr(), buf.len()))
     }
 }
 
@@ -161,10 +158,6 @@ impl DateTimeLike for DateTime {
     pydatetime_get!(second, PyDateTime_DATE_GET_SECOND, u8);
     pydatetime_get!(microsecond, PyDateTime_DATE_GET_MICROSECOND, u32);
 
-    fn millisecond(&self) -> u32 {
-        self.microsecond() / 1_000
-    }
-
     fn nanosecond(&self) -> u32 {
         self.microsecond() * 1_000
     }
@@ -241,15 +234,14 @@ impl DateTimeLike for DateTime {
 }
 
 impl Serialize for DateTime {
-    #[inline(never)]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
     {
-        let mut buf = DateTimeBuffer::new();
+        let mut buf = SmallFixedBuffer::new();
         if self.write_buf(&mut buf, self.opts).is_err() {
             err!(SerializeError::DatetimeLibraryUnsupported)
         }
-        serializer.serialize_str(str_from_slice!(buf.as_ptr(), buf.len()))
+        serializer.serialize_unit_struct(str_from_slice!(buf.as_ptr(), buf.len()))
     }
 }
