@@ -1,17 +1,18 @@
 use crate::opt::*;
 
+use crate::serialize::buffer::SmallFixedBuffer;
 use crate::serialize::error::SerializeError;
 use crate::serialize::per_type::{
-    DateTimeBuffer, DateTimeError, DateTimeLike, DefaultSerializer, Offset, ZeroListSerializer,
+    DateTimeError, DateTimeLike, DefaultSerializer, Offset, ZeroListSerializer,
 };
 use crate::serialize::serializer::PyObjectSerializer;
 use crate::typeref::{load_numpy_types, ARRAY_STRUCT_STR, DESCR_STR, DTYPE_STR, NUMPY_TYPES};
-use chrono::{Datelike, NaiveDate, NaiveDateTime, Timelike};
+use core::ffi::{c_char, c_int, c_void};
+use jiff::civil::DateTime;
+use jiff::Timestamp;
 use pyo3_ffi::*;
 use serde::ser::{self, Serialize, SerializeSeq, Serializer};
-use std::convert::TryInto;
 use std::fmt;
-use std::os::raw::{c_char, c_int, c_void};
 
 #[repr(transparent)]
 pub struct NumpySerializer<'a> {
@@ -43,6 +44,9 @@ impl<'a> Serialize for NumpySerializer<'a> {
             Err(PyArrayError::NotContiguous) => {
                 err!(SerializeError::NumpyNotCContiguous)
             }
+            Err(PyArrayError::NotNativeEndian) => {
+                err!(SerializeError::NumpyNotNativeEndian)
+            }
             Err(PyArrayError::UnsupportedDataType) => {
                 err!(SerializeError::NumpyUnsupportedDatatype)
             }
@@ -52,7 +56,7 @@ impl<'a> Serialize for NumpySerializer<'a> {
 
 macro_rules! slice {
     ($ptr:expr, $size:expr) => {
-        unsafe { std::slice::from_raw_parts($ptr, $size) }
+        unsafe { core::slice::from_raw_parts($ptr, $size) }
     };
 }
 
@@ -65,6 +69,7 @@ pub fn is_numpy_scalar(ob_type: *mut PyTypeObject) -> bool {
         let scalar_types = unsafe { numpy_types.unwrap().as_ref() };
         ob_type == scalar_types.float64
             || ob_type == scalar_types.float32
+            || ob_type == scalar_types.float16
             || ob_type == scalar_types.int64
             || ob_type == scalar_types.int16
             || ob_type == scalar_types.int32
@@ -101,6 +106,9 @@ pub struct PyCapsule {
 
 // https://docs.scipy.org/doc/numpy/reference/arrays.interface.html#c.__array_struct__
 
+const NPY_ARRAY_C_CONTIGUOUS: c_int = 0x1;
+const NPY_ARRAY_NOTSWAPPED: c_int = 0x200;
+
 #[repr(C)]
 pub struct PyArrayInterface {
     pub two: c_int,
@@ -118,6 +126,7 @@ pub struct PyArrayInterface {
 pub enum ItemType {
     BOOL,
     DATETIME64(NumpyDatetimeUnit),
+    F16,
     F32,
     F64,
     I8,
@@ -138,6 +147,7 @@ impl ItemType {
                 let unit = NumpyDatetimeUnit::from_pyobject(ptr);
                 Some(ItemType::DATETIME64(unit))
             }
+            (102, 2) => Some(ItemType::F16),
             (102, 4) => Some(ItemType::F32),
             (102, 8) => Some(ItemType::F64),
             (105, 1) => Some(ItemType::I8),
@@ -152,9 +162,11 @@ impl ItemType {
         }
     }
 }
+
 pub enum PyArrayError {
     Malformed,
     NotContiguous,
+    NotNativeEndian,
     UnsupportedDataType,
 }
 
@@ -185,9 +197,12 @@ impl NumpyArray {
         if unsafe { (*array).two != 2 } {
             ffi!(Py_DECREF(capsule));
             Err(PyArrayError::Malformed)
-        } else if unsafe { (*array).flags } & 0x1 != 0x1 {
+        } else if unsafe { (*array).flags } & NPY_ARRAY_C_CONTIGUOUS != NPY_ARRAY_C_CONTIGUOUS {
             ffi!(Py_DECREF(capsule));
             Err(PyArrayError::NotContiguous)
+        } else if unsafe { (*array).flags } & NPY_ARRAY_NOTSWAPPED != NPY_ARRAY_NOTSWAPPED {
+            ffi!(Py_DECREF(capsule));
+            Err(PyArrayError::NotNativeEndian)
         } else {
             let num_dimensions = unsafe { (*array).nd as usize };
             if num_dimensions == 0 {
@@ -236,7 +251,7 @@ impl NumpyArray {
     #[cfg_attr(feature = "optimize", optimize(size))]
     fn build(&mut self) {
         if self.depth < self.dimensions() - 1 {
-            for i in 0..=self.shape()[self.depth] - 1 {
+            for i in 0..self.shape()[self.depth] {
                 let mut position: Vec<isize> = self.position.to_vec();
                 position[self.depth] = i;
                 let num_children: usize = if self.depth < self.dimensions() - 2 {
@@ -250,6 +265,7 @@ impl NumpyArray {
         }
     }
 
+    #[inline(always)]
     fn data(&self) -> *const c_void {
         let offset = self
             .strides()
@@ -313,6 +329,10 @@ impl Serialize for NumpyArray {
                     NumpyF32Array::new(slice!(self.data() as *const f32, self.num_items()))
                         .serialize(serializer)
                 }
+                ItemType::F16 => {
+                    NumpyF16Array::new(slice!(self.data() as *const u16, self.num_items()))
+                        .serialize(serializer)
+                }
                 ItemType::U64 => {
                     NumpyU64Array::new(slice!(self.data() as *const u64, self.num_items()))
                         .serialize(serializer)
@@ -373,6 +393,7 @@ impl<'a> NumpyF64Array<'a> {
 
 impl<'a> Serialize for NumpyF64Array<'a> {
     #[cold]
+    #[inline(never)]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -413,6 +434,7 @@ impl<'a> NumpyF32Array<'a> {
 
 impl<'a> Serialize for NumpyF32Array<'a> {
     #[cold]
+    #[inline(never)]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -440,6 +462,49 @@ impl Serialize for DataTypeF32 {
     }
 }
 
+#[repr(transparent)]
+struct NumpyF16Array<'a> {
+    data: &'a [u16],
+}
+
+impl<'a> NumpyF16Array<'a> {
+    fn new(data: &'a [u16]) -> Self {
+        Self { data }
+    }
+}
+
+impl<'a> Serialize for NumpyF16Array<'a> {
+    #[cold]
+    #[inline(never)]
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        let mut seq = serializer.serialize_seq(None).unwrap();
+        for &each in self.data.iter() {
+            seq.serialize_element(&DataTypeF16 { obj: each }).unwrap();
+        }
+        seq.end()
+    }
+}
+
+#[repr(transparent)]
+struct DataTypeF16 {
+    obj: u16,
+}
+
+impl Serialize for DataTypeF16 {
+    #[cold]
+    #[cfg_attr(feature = "optimize", optimize(size))]
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        let as_f16 = half::f16::from_bits(self.obj);
+        serializer.serialize_f32(as_f16.to_f32())
+    }
+}
+
 #[repr(transparent)]
 struct NumpyU64Array<'a> {
     data: &'a [u64],
@@ -453,6 +518,7 @@ impl<'a> NumpyU64Array<'a> {
 
 impl<'a> Serialize for NumpyU64Array<'a> {
     #[cold]
+    #[inline(never)]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -493,6 +559,7 @@ impl<'a> NumpyU32Array<'a> {
 
 impl<'a> Serialize for NumpyU32Array<'a> {
     #[cold]
+    #[inline(never)]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -533,6 +600,7 @@ impl<'a> NumpyU16Array<'a> {
 
 impl<'a> Serialize for NumpyU16Array<'a> {
     #[cold]
+    #[inline(never)]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -556,7 +624,7 @@ impl Serialize for DataTypeU16 {
     where
         S: Serializer,
     {
-        serializer.serialize_u16(self.obj)
+        serializer.serialize_u32(self.obj as u32)
     }
 }
 
@@ -573,6 +641,7 @@ impl<'a> NumpyI64Array<'a> {
 
 impl<'a> Serialize for NumpyI64Array<'a> {
     #[cold]
+    #[inline(never)]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -613,6 +682,7 @@ impl<'a> NumpyI32Array<'a> {
 
 impl<'a> Serialize for NumpyI32Array<'a> {
     #[cold]
+    #[inline(never)]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -653,6 +723,7 @@ impl<'a> NumpyI16Array<'a> {
 
 impl<'a> Serialize for NumpyI16Array<'a> {
     #[cold]
+    #[inline(never)]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -676,7 +747,7 @@ impl Serialize for DataTypeI16 {
     where
         S: Serializer,
     {
-        serializer.serialize_i16(self.obj)
+        serializer.serialize_i32(self.obj as i32)
     }
 }
 
@@ -693,6 +764,7 @@ impl<'a> NumpyI8Array<'a> {
 
 impl<'a> Serialize for NumpyI8Array<'a> {
     #[cold]
+    #[inline(never)]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -716,7 +788,7 @@ impl Serialize for DataTypeI8 {
     where
         S: Serializer,
     {
-        serializer.serialize_i8(self.obj)
+        serializer.serialize_i32(self.obj as i32)
     }
 }
 
@@ -733,6 +805,7 @@ impl<'a> NumpyU8Array<'a> {
 
 impl<'a> Serialize for NumpyU8Array<'a> {
     #[cold]
+    #[inline(never)]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -756,7 +829,7 @@ impl Serialize for DataTypeU8 {
     where
         S: Serializer,
     {
-        serializer.serialize_u8(self.obj)
+        serializer.serialize_u32(self.obj as u32)
     }
 }
 
@@ -827,6 +900,8 @@ impl Serialize for NumpyScalar {
                 (*(self.ptr as *mut NumpyFloat64)).serialize(serializer)
             } else if ob_type == scalar_types.float32 {
                 (*(self.ptr as *mut NumpyFloat32)).serialize(serializer)
+            } else if ob_type == scalar_types.float16 {
+                (*(self.ptr as *mut NumpyFloat16)).serialize(serializer)
             } else if ob_type == scalar_types.int64 {
                 (*(self.ptr as *mut NumpyInt64)).serialize(serializer)
             } else if ob_type == scalar_types.int32 {
@@ -872,7 +947,7 @@ impl Serialize for NumpyInt8 {
     where
         S: Serializer,
     {
-        serializer.serialize_i8(self.value)
+        serializer.serialize_i32(self.value as i32)
     }
 }
 
@@ -889,7 +964,7 @@ impl Serialize for NumpyInt16 {
     where
         S: Serializer,
     {
-        serializer.serialize_i16(self.value)
+        serializer.serialize_i32(self.value as i32)
     }
 }
 
@@ -940,7 +1015,7 @@ impl Serialize for NumpyUint8 {
     where
         S: Serializer,
     {
-        serializer.serialize_u8(self.value)
+        serializer.serialize_u32(self.value as u32)
     }
 }
 
@@ -957,7 +1032,7 @@ impl Serialize for NumpyUint16 {
     where
         S: Serializer,
     {
-        serializer.serialize_u16(self.value)
+        serializer.serialize_u32(self.value as u32)
     }
 }
 
@@ -995,6 +1070,24 @@ impl Serialize for NumpyUint64 {
     }
 }
 
+#[repr(C)]
+pub struct NumpyFloat16 {
+    ob_refcnt: Py_ssize_t,
+    ob_type: *mut PyTypeObject,
+    value: u16,
+}
+
+impl Serialize for NumpyFloat16 {
+    #[cold]
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        let as_f16 = half::f16::from_bits(self.value);
+        serializer.serialize_f32(as_f16.to_f32())
+    }
+}
+
 #[repr(C)]
 pub struct NumpyFloat32 {
     ob_refcnt: Py_ssize_t,
@@ -1112,6 +1205,19 @@ impl NumpyDateTimeError {
     }
 }
 
+macro_rules! to_jiff_datetime {
+    ($timestamp:expr, $self:expr, $val:expr) => {
+        Ok(
+            ($timestamp.map_err(|_| NumpyDateTimeError::Unrepresentable {
+                unit: *$self,
+                val: $val,
+            })?)
+            .to_zoned(jiff::tz::TimeZone::UTC)
+            .datetime(),
+        )
+    };
+}
+
 impl NumpyDatetimeUnit {
     /// Create a `NumpyDatetimeUnit` from a pointer to a Python object holding a
     /// numpy array.
@@ -1164,17 +1270,19 @@ impl NumpyDatetimeUnit {
     #[cfg_attr(feature = "optimize", optimize(size))]
     fn datetime(&self, val: i64, opts: Opt) -> Result<NumpyDatetime64Repr, NumpyDateTimeError> {
         match self {
-            Self::Years => Ok(NaiveDate::from_ymd_opt(
+            Self::Years => Ok(DateTime::new(
                 (val + 1970)
                     .try_into()
                     .map_err(|_| NumpyDateTimeError::Unrepresentable { unit: *self, val })?,
                 1,
                 1,
+                0,
+                0,
+                0,
+                0,
             )
-            .unwrap()
-            .and_hms_opt(0, 0, 0)
             .unwrap()),
-            Self::Months => Ok(NaiveDate::from_ymd_opt(
+            Self::Months => Ok(DateTime::new(
                 (val / 12 + 1970)
                     .try_into()
                     .map_err(|_| NumpyDateTimeError::Unrepresentable { unit: *self, val })?,
@@ -1182,38 +1290,24 @@ impl NumpyDatetimeUnit {
                     .try_into()
                     .map_err(|_| NumpyDateTimeError::Unrepresentable { unit: *self, val })?,
                 1,
+                0,
+                0,
+                0,
+                0,
             )
-            .unwrap()
-            .and_hms_opt(0, 0, 0)
             .unwrap()),
             Self::Weeks => {
-                Ok(NaiveDateTime::from_timestamp_opt(val * 7 * 24 * 60 * 60, 0).unwrap())
+                to_jiff_datetime!(Timestamp::from_second(val * 7 * 24 * 60 * 60), self, val)
+            }
+            Self::Days => to_jiff_datetime!(Timestamp::from_second(val * 24 * 60 * 60), self, val),
+            Self::Hours => to_jiff_datetime!(Timestamp::from_second(val * 60 * 60), self, val),
+            Self::Minutes => to_jiff_datetime!(Timestamp::from_second(val * 60), self, val),
+            Self::Seconds => to_jiff_datetime!(Timestamp::from_second(val), self, val),
+            Self::Milliseconds => to_jiff_datetime!(Timestamp::from_millisecond(val), self, val),
+            Self::Microseconds => to_jiff_datetime!(Timestamp::from_microsecond(val), self, val),
+            Self::Nanoseconds => {
+                to_jiff_datetime!(Timestamp::from_nanosecond(val as i128), self, val)
             }
-            Self::Days => Ok(NaiveDateTime::from_timestamp_opt(val * 24 * 60 * 60, 0).unwrap()),
-            Self::Hours => Ok(NaiveDateTime::from_timestamp_opt(val * 60 * 60, 0).unwrap()),
-            Self::Minutes => Ok(NaiveDateTime::from_timestamp_opt(val * 60, 0).unwrap()),
-            Self::Seconds => Ok(NaiveDateTime::from_timestamp_opt(val, 0).unwrap()),
-            Self::Milliseconds => Ok(NaiveDateTime::from_timestamp_opt(
-                val / 1_000,
-                (val % 1_000 * 1_000_000)
-                    .try_into()
-                    .map_err(|_| NumpyDateTimeError::Unrepresentable { unit: *self, val })?,
-            )
-            .unwrap()),
-            Self::Microseconds => Ok(NaiveDateTime::from_timestamp_opt(
-                val / 1_000_000,
-                (val % 1_000_000 * 1_000)
-                    .try_into()
-                    .map_err(|_| NumpyDateTimeError::Unrepresentable { unit: *self, val })?,
-            )
-            .unwrap()),
-            Self::Nanoseconds => Ok(NaiveDateTime::from_timestamp_opt(
-                val / 1_000_000_000,
-                (val % 1_000_000_000)
-                    .try_into()
-                    .map_err(|_| NumpyDateTimeError::Unrepresentable { unit: *self, val })?,
-            )
-            .unwrap()),
             _ => Err(NumpyDateTimeError::UnsupportedUnit(*self)),
         }
         .map(|dt| NumpyDatetime64Repr { dt, opts })
@@ -1266,7 +1360,7 @@ macro_rules! forward_inner {
 }
 
 struct NumpyDatetime64Repr {
-    dt: NaiveDateTime,
+    dt: DateTime,
     opts: Opt,
 }
 
@@ -1277,10 +1371,9 @@ impl DateTimeLike for NumpyDatetime64Repr {
     forward_inner!(hour, u8);
     forward_inner!(minute, u8);
     forward_inner!(second, u8);
-    forward_inner!(nanosecond, u32);
 
-    fn millisecond(&self) -> u32 {
-        self.nanosecond() / 1_000_000
+    fn nanosecond(&self) -> u32 {
+        self.dt.subsec_nanosecond() as u32
     }
 
     fn microsecond(&self) -> u32 {
@@ -1306,7 +1399,7 @@ impl Serialize for NumpyDatetime64Repr {
     where
         S: Serializer,
     {
-        let mut buf = DateTimeBuffer::new();
+        let mut buf = SmallFixedBuffer::new();
         let _ = self.write_buf(&mut buf, self.opts);
         serializer.collect_str(str_from_slice!(buf.as_ptr(), buf.len()))
     }
