@@ -20,9 +20,27 @@ macro_rules! is_class_by_type {
     };
 }
 
+#[cfg(not(Py_GIL_DISABLED))]
+macro_rules! tp_flags {
+    ($ob_type:expr) => {
+        unsafe { (*$ob_type).tp_flags }
+    };
+}
+
+#[cfg(Py_GIL_DISABLED)]
+macro_rules! tp_flags {
+    ($ob_type:expr) => {
+        unsafe {
+            (*$ob_type)
+                .tp_flags
+                .load(std::sync::atomic::Ordering::Relaxed)
+        }
+    };
+}
+
 macro_rules! is_subclass_by_flag {
-    ($ob_type:expr, $flag:ident) => {
-        unsafe { (((*$ob_type).tp_flags & pyo3_ffi::$flag) != 0) }
+    ($tp_flags:expr, $flag:ident) => {
+        unsafe { (($tp_flags & pyo3_ffi::$flag) != 0) }
     };
 }
 
@@ -59,7 +77,7 @@ macro_rules! opt_disabled {
 #[cfg(feature = "intrinsics")]
 macro_rules! unlikely {
     ($exp:expr) => {
-        std::intrinsics::unlikely($exp)
+        core::intrinsics::unlikely($exp)
     };
 }
 
@@ -74,7 +92,7 @@ macro_rules! unlikely {
 #[cfg(feature = "intrinsics")]
 macro_rules! likely {
     ($exp:expr) => {
-        std::intrinsics::likely($exp)
+        core::intrinsics::likely($exp)
     };
 }
 
@@ -88,17 +106,17 @@ macro_rules! likely {
 
 macro_rules! nonnull {
     ($exp:expr) => {
-        unsafe { std::ptr::NonNull::new_unchecked($exp) }
+        unsafe { core::ptr::NonNull::new_unchecked($exp) }
     };
 }
 
 macro_rules! str_from_slice {
     ($ptr:expr, $size:expr) => {
-        unsafe { std::str::from_utf8_unchecked(std::slice::from_raw_parts($ptr, $size as usize)) }
+        unsafe { std::str::from_utf8_unchecked(core::slice::from_raw_parts($ptr, $size as usize)) }
     };
 }
 
-#[cfg(Py_3_12)]
+#[cfg(all(Py_3_12, not(Py_GIL_DISABLED)))]
 macro_rules! reverse_pydict_incref {
     ($op:expr) => {
         unsafe {
@@ -110,6 +128,14 @@ macro_rules! reverse_pydict_incref {
     };
 }
 
+#[cfg(Py_GIL_DISABLED)]
+macro_rules! reverse_pydict_incref {
+    ($op:expr) => {
+        debug_assert!(ffi!(Py_REFCNT($op)) >= 2);
+        ffi!(Py_DECREF($op))
+    };
+}
+
 #[cfg(not(Py_3_12))]
 macro_rules! reverse_pydict_incref {
     ($op:expr) => {
@@ -159,7 +185,7 @@ macro_rules! call_method {
             pyo3_ffi::PyObject_CallMethodObjArgs(
                 $obj1,
                 $obj2,
-                std::ptr::null_mut() as *mut pyo3_ffi::PyObject,
+                core::ptr::null_mut() as *mut pyo3_ffi::PyObject,
             )
         }
     };
@@ -169,7 +195,7 @@ macro_rules! call_method {
                 $obj1,
                 $obj2,
                 $obj3,
-                std::ptr::null_mut() as *mut pyo3_ffi::PyObject,
+                core::ptr::null_mut() as *mut pyo3_ffi::PyObject,
             )
         }
     };
@@ -238,9 +264,17 @@ macro_rules! use_immortal {
     };
 }
 
+#[cfg(not(Py_3_13))]
 macro_rules! pydict_next {
     ($obj1:expr, $obj2:expr, $obj3:expr, $obj4:expr) => {
-        unsafe { pyo3_ffi::_PyDict_Next($obj1, $obj2, $obj3, $obj4, std::ptr::null_mut()) }
+        unsafe { pyo3_ffi::_PyDict_Next($obj1, $obj2, $obj3, $obj4, core::ptr::null_mut()) }
+    };
+}
+
+#[cfg(Py_3_13)]
+macro_rules! pydict_next {
+    ($obj1:expr, $obj2:expr, $obj3:expr, $obj4:expr) => {
+        unsafe { pyo3_ffi::PyDict_Next($obj1, $obj2, $obj3, $obj4) }
     };
 }
 
@@ -249,3 +283,57 @@ macro_rules! reserve_minimum {
         $writer.reserve(64);
     };
 }
+
+macro_rules! reserve_pretty {
+    ($writer:expr, $val:expr) => {
+        $writer.reserve($val + 16);
+    };
+}
+
+macro_rules! assume {
+    ($expr:expr) => {
+        debug_assert!($expr);
+        #[cfg(feature = "intrinsics")]
+        unsafe {
+            core::intrinsics::assume($expr);
+        };
+    };
+}
+
+#[allow(unused_macros)]
+#[cfg(feature = "intrinsics")]
+macro_rules! trailing_zeros {
+    ($val:expr) => {
+        core::intrinsics::cttz_nonzero($val) as usize
+    };
+}
+
+#[allow(unused_macros)]
+#[cfg(not(feature = "intrinsics"))]
+macro_rules! trailing_zeros {
+    ($val:expr) => {
+        $val.trailing_zeros() as usize
+    };
+}
+
+#[allow(unused_macros)]
+#[cfg(feature = "intrinsics")]
+macro_rules! popcnt {
+    ($val:expr) => {
+        core::intrinsics::ctpop(core::mem::transmute::<u32, i32>($val)) as usize
+    };
+}
+
+#[allow(unused_macros)]
+#[cfg(not(feature = "intrinsics"))]
+macro_rules! popcnt {
+    ($val:expr) => {
+        core::mem::transmute::<u32, i32>($val).count_ones() as usize
+    };
+}
+
+macro_rules! unreachable_unchecked {
+    () => {
+        unsafe { core::hint::unreachable_unchecked() }
+    };
+}
